@@ -1,14 +1,17 @@
 """Augmentations."""
+#  pylint: disable=invalid-name
 from pylint.checkers.base import DocStringChecker, NameChecker
 from pylint.checkers.design_analysis import MisdesignChecker
 from pylint.checkers.classes import ClassChecker
 from pylint.checkers.newstyle import NewStyleConflictChecker
 from pylint.checkers.variables import VariablesChecker
-from astroid import InferenceError, Getattr
-from astroid.nodes import Class, From
+from astroid import InferenceError
+from pylint_django.compat import ClassDef, ImportFrom, Attribute, django_version
 from astroid.scoped_nodes import Class as ScopedClass, Module
+from pylint.__pkginfo__ import numversion as PYLINT_VERSION
 from pylint.checkers.typecheck import TypeChecker
 from pylint_django.utils import node_is_subclass, PY3
+from pylint_django.compat import inferred
 from pylint_plugin_utils import augment_visit, suppress_message
 
 
@@ -30,7 +33,7 @@ def ignore_import_warnings_for_related_fields(orig_method, self, node):
 
     iterat = to_consume[0].items if PY3 else to_consume[0].iteritems
     for name, stmts in iterat():
-        if isinstance(stmts[0], From):
+        if isinstance(stmts[0], ImportFrom):
             if any([n[0] in ('ForeignKey', 'OneToOneField') for n in stmts[0].names]):
                 continue
         new_things[name] = stmts
@@ -63,55 +66,53 @@ def foreign_key_sets(chain, node):
     warn.
     """
     quack = False
+    # Note: it would have been nice to import the Manager object from Django and
+    # get its attributes that way - and this used to be the method - but unfortunately
+    # there's no guarantee that Django is properly configured at that stage, and importing
+    # anything from the django.db package causes an ImproperlyConfigured exception.
+    # Therefore we'll fall back on a hard-coded list of attributes which won't be as accurate,
+    # but this is not 100% accurate anyway.
+    manager_attrs = (
+        'none',
+        'all',
+        'count',
+        'dates',
+        'distinct',
+        'extra',
+        'get',
+        'get_or_create',
+        'create',
+        'bulk_create',
+        'filter',
+        'aggregate',
+        'annotate',
+        'complex_filter',
+        'exclude',
+        'in_bulk',
+        'iterator',
+        'latest',
+        'order_by',
+        'select_for_update',
+        'select_related',
+        'prefetch_related',
+        'values',
+        'values_list',
+        'update',
+        'reverse',
+        'defer',
+        'only',
+        'using',
+        'exists',
+    )
 
-    if node.attrname.endswith('_set'):
+    if node.attrname in manager_attrs or node.attrname.endswith('_set'):
         # if this is a X_set method, that's a pretty strong signal that this is the default
         # Django name, rather than one set by related_name
         quack = True
     else:
         # we will
-        if isinstance(node.parent, Getattr):
+        if isinstance(node.parent, Attribute):
             func_name = getattr(node.parent, 'attrname', None)
-
-            # Note: it would have been nice to import the Manager object from Django and
-            # get its attributes that way - and this used to be the method - but unfortunately
-            # there's no guarantee that Django is properly configured at that stage, and importing
-            # anything from the django.db package causes an ImproperlyConfigured exception.
-            # Therefore we'll fall back on a hard-coded list of attributes which won't be as accurate,
-            # but this is not 100% accurate anyway.
-            manager_attrs = (
-                'none',
-                'all',
-                'count',
-                'dates',
-                'distinct',
-                'extra',
-                'get',
-                'get_or_create',
-                'create',
-                'bulk_create',
-                'filter',
-                'aggregate',
-                'annotate',
-                'complex_filter',
-                'exclude',
-                'in_bulk',
-                'iterator',
-                'latest',
-                'order_by',
-                'select_for_update',
-                'select_related',
-                'prefetch_related',
-                'values',
-                'values_list',
-                'update',
-                'reverse',
-                'defer',
-                'only',
-                'using',
-                'exists',
-            )
-
             if func_name in manager_attrs:
                 quack = True
 
@@ -119,12 +120,15 @@ def foreign_key_sets(chain, node):
         children = list(node.get_children())
         for child in children:
             try:
-                inferred = child.infered()
+                inferred_cls = inferred(child)()
             except InferenceError:
                 pass
             else:
-                for cls in inferred:
-                    if node_is_subclass(cls, 'django.db.models.base.Model'):
+                for cls in inferred_cls:
+                    if (node_is_subclass(cls,
+                                         'django.db.models.manager.Manager',
+                                         'django.db.models.base.Model',
+                                         '.Model')):
                         # This means that we are looking at a subclass of models.Model
                         # and something is trying to access a <something>_set attribute.
                         # Since this could exist, we will return so as not to raise an
@@ -133,9 +137,15 @@ def foreign_key_sets(chain, node):
     chain()
 
 
+def foreign_key_ids(chain, node):
+    if node.attrname.endswith('_id'):
+        return
+    chain()
+
+
 def is_model_admin_subclass(node):
     """Checks that node is derivative of ModelAdmin class."""
-    if node.name[-5:] != 'Admin' or isinstance(node.parent, Class):
+    if node.name[-5:] != 'Admin' or isinstance(node.parent, ClassDef):
         return False
 
     return node_is_subclass(node, 'django.contrib.admin.options.ModelAdmin')
@@ -143,42 +153,58 @@ def is_model_admin_subclass(node):
 
 def is_model_media_subclass(node):
     """Checks that node is derivative of Media class."""
-    if node.name != 'Media' or not isinstance(node.parent, Class):
+    if node.name != 'Media' or not isinstance(node.parent, ClassDef):
         return False
 
     parents = ('django.contrib.admin.options.ModelAdmin',
                'django.forms.widgets.Media',
                'django.db.models.base.Model',
+               '.Model',  # for the transformed version used in this plugin
                'django.forms.forms.Form',
-               'django.forms.models.ModelForm')
-    return any([node_is_subclass(node.parent, parent) for parent in parents])
+               '.Form',
+               'django.forms.widgets.Widget',
+               '.Widget',
+               'django.forms.models.ModelForm',
+               '.ModelForm')
+    return node_is_subclass(node.parent, *parents)
 
 
 def is_model_meta_subclass(node):
     """Checks that node is derivative of Meta class."""
-    if node.name != 'Meta' or not isinstance(node.parent, Class):
+    if node.name != 'Meta' or not isinstance(node.parent, ClassDef):
         return False
 
-    parents = ('django.db.models.base.Model',
+    parents = ('.Model',  # for the transformed version used here
+               'django.db.models.base.Model',
+               '.Form',
                'django.forms.forms.Form',
-               'django.forms.models.ModelForm')
-    return any([node_is_subclass(node.parent, parent) for parent in parents])
+               '.ModelForm',
+               'django.forms.models.ModelForm',
+               'rest_framework.serializers.ModelSerializer',
+               'rest_framework.generics.GenericAPIView',
+               'rest_framework.viewsets.ReadOnlyModelViewSet',
+               'rest_framework.viewsets.ModelViewSet',
+               'django_filters.filterset.FilterSet',)
+    return node_is_subclass(node.parent, *parents)
 
 
 def is_model_mpttmeta_subclass(node):
     """Checks that node is derivative of MPTTMeta class."""
-    if node.name != 'MPTTMeta' or not isinstance(node.parent, Class):
+    if node.name != 'MPTTMeta' or not isinstance(node.parent, ClassDef):
         return False
 
     parents = ('django.db.models.base.Model',
+               '.Model',  # for the transformed version used in this plugin
                'django.forms.forms.Form',
-               'django.forms.models.ModelForm')
-    return any([node_is_subclass(node.parent, parent) for parent in parents])
+               '.Form',
+               'django.forms.models.ModelForm',
+               '.ModelForm')
+    return node_is_subclass(node.parent, *parents)
 
 
 def is_model_test_case_subclass(node):
     """Checks that node is derivative of TestCase class."""
-    if not node.name.endswith('Test') and not isinstance(node.parent, Class):
+    if not node.name.endswith('Test') and not isinstance(node.parent, ClassDef):
         return False
 
     return node_is_subclass(node, 'django.test.testcases.TestCase')
@@ -220,8 +246,8 @@ def is_model_field_display_method(node):
         # TODO: could validate the names of the fields on the model rather than
         # blindly accepting get_*_display
         try:
-            for cls in node.last_child().infered():
-                if node_is_subclass(cls, 'django.db.models.base.Model'):
+            for cls in inferred(node.last_child())():
+                if node_is_subclass(cls, 'django.db.models.base.Model', '.Model'):
                     return True
         except InferenceError:
             return False
@@ -260,7 +286,7 @@ def is_templatetags_module_valid_constant(node):
 
 def is_urls_module_valid_constant(node):
     """Suppress warnings for valid constants in urls module."""
-    if node.name not in ('urlpatterns', ):
+    if node.name not in ('urlpatterns', 'app_name'):
         return False
 
     parent = node.parent
@@ -273,6 +299,13 @@ def is_urls_module_valid_constant(node):
     return True
 
 
+def allow_meta_protected_access(node):
+    if django_version >= (1, 8):
+        return node.attrname == '_meta'
+    else:
+        return False
+
+
 def is_class(class_name):
     """Shortcut for node_is_subclass."""
     return lambda node: node_is_subclass(node, class_name)
@@ -284,41 +317,71 @@ def wrap(orig_method, with_method):
     return wrap_func
 
 
+# The names of some visit functions changed in this commit:
+# https://bitbucket.org/logilab/pylint/commits/c94ee95abaa5737f13b91626fe321150c0ddd140
+
+def _visit_class(checker):
+    return getattr(checker, 'visit_classdef' if PYLINT_VERSION >= (1, 5) else 'visit_class')
+
+
+def _visit_attribute(checker):
+    return getattr(checker, 'visit_attribute' if PYLINT_VERSION >= (1, 5) else 'visit_getattr')
+
+
+def _leave_class(checker):
+    return getattr(checker, 'leave_classdef' if PYLINT_VERSION >= (1, 5) else 'leave_class')
+
+
+def _leave_function(checker):
+    return getattr(checker, 'leave_functiondef' if PYLINT_VERSION >= (1, 5) else 'leave_function')
+
+
+def _visit_assignname(checker):
+    return getattr(checker, 'visit_assignname' if PYLINT_VERSION >= (1, 5) else 'visit_assname')
+
+
+def _visit_assign(checker):
+    return getattr(checker, 'visit_assign')
+
+
 def apply_augmentations(linter):
     """Apply augmentation and suppression rules."""
-    augment_visit(linter, TypeChecker.visit_getattr, foreign_key_sets)
-    suppress_message(linter, TypeChecker.visit_getattr, 'E1101', is_model_field_display_method)
+    augment_visit(linter, _visit_attribute(TypeChecker), foreign_key_sets)
+    augment_visit(linter, _visit_attribute(TypeChecker), foreign_key_ids)
+    suppress_message(linter, _visit_attribute(TypeChecker), 'E1101', is_model_field_display_method)
 
     # formviews have too many ancestors, there's nothing the user of the library can do about that
-    suppress_message(linter, MisdesignChecker.visit_class, 'R0901', is_class('django.views.generic.edit.FormView'))
+    suppress_message(linter, _visit_class(MisdesignChecker), 'R0901', is_class('django.views.generic.edit.FormView'))
 
     # model forms have no __init__ method anywhere in their bases
-    suppress_message(linter, ClassChecker.visit_class, 'W0232', is_class('django.forms.models.ModelForm'))
+    suppress_message(linter, _visit_class(ClassChecker), 'W0232', is_class('django.forms.models.ModelForm'))
 
     # forms implement __getitem__ but not __len__, thus raising a "Badly implemented container" warning which
     # we will suppress.
-    suppress_message(linter, MisdesignChecker.leave_class, 'R0924', is_class('django.forms.forms.Form'))
-    suppress_message(linter, MisdesignChecker.leave_class, 'R0924', is_class('django.forms.models.ModelForm'))
+    suppress_message(linter, _leave_class(MisdesignChecker), 'R0924', is_class('django.forms.forms.Form'))
+    suppress_message(linter, _leave_class(MisdesignChecker), 'R0924', is_class('django.forms.models.ModelForm'))
 
     # Meta
-    suppress_message(linter, DocStringChecker.visit_class, 'C0111', is_model_meta_subclass)
-    suppress_message(linter, NewStyleConflictChecker.visit_class, 'C1001', is_model_meta_subclass)
-    suppress_message(linter, ClassChecker.visit_class, 'W0232', is_model_meta_subclass)
-    suppress_message(linter, MisdesignChecker.leave_class, 'R0903', is_model_meta_subclass)
+    suppress_message(linter, _visit_class(DocStringChecker), 'missing-docstring', is_model_meta_subclass)
+    suppress_message(linter, _visit_class(NewStyleConflictChecker), 'old-style-class', is_model_meta_subclass)
+    suppress_message(linter, _visit_class(ClassChecker), 'no-init', is_model_meta_subclass)
+    suppress_message(linter, _leave_class(MisdesignChecker), 'too-few-public-methods', is_model_meta_subclass)
+    suppress_message(linter, _visit_attribute(ClassChecker), 'protected-access', allow_meta_protected_access)
 
     # Media
-    suppress_message(linter, NameChecker.visit_assname, 'C0103', is_model_media_valid_attributes)
-    suppress_message(linter, DocStringChecker.visit_class, 'C0111', is_model_media_subclass)
-    suppress_message(linter, NewStyleConflictChecker.visit_class, 'C1001', is_model_media_subclass)
-    suppress_message(linter, ClassChecker.visit_class, 'W0232', is_model_media_subclass)
-    suppress_message(linter, MisdesignChecker.leave_class, 'R0903', is_model_media_subclass)
+    suppress_message(linter, _visit_assignname(NameChecker), 'C0103', is_model_media_valid_attributes)
+    suppress_message(linter, _visit_class(DocStringChecker), 'missing-docstring', is_model_media_subclass)
+    suppress_message(linter, _visit_class(NewStyleConflictChecker), 'old-style-class', is_model_media_subclass)
+    suppress_message(linter, _visit_class(ClassChecker), 'no-init', is_model_media_subclass)
+    suppress_message(linter, _leave_class(MisdesignChecker), 'too-few-public-methods', is_model_media_subclass)
 
     # Too few public methods started appearing for Views and Models as part of Pylint>=1.4 / astroid>=1.3.3
     # Not sure why, suspect this is a failure to get the parent classes somewhere
     # For now, just suppress it on models and views
-    suppress_message(linter, MisdesignChecker.leave_class, 'R0903', is_class('django.db.models.base.Model'))
-    # TODO: why does this not work with the fqn of 'View'? Must be something to do with the overriding and transforms
-    suppress_message(linter, MisdesignChecker.leave_class, 'R0903', is_class('.View'))
+    suppress_message(linter, _leave_class(MisdesignChecker), 'too-few-public-methods',
+                     is_class('.Model'))
+    suppress_message(linter, _leave_class(MisdesignChecker), 'too-few-public-methods',
+                     is_class('.View'))
 
     # Admin
     # Too many public methods (40+/20)
@@ -328,22 +391,22 @@ def apply_augmentations(linter):
     #for method in node.methods():
     #    if not method.name.startswith('_'):
     #        nb_public_methods += 1
-    suppress_message(linter, MisdesignChecker.leave_class, 'R0904', is_model_admin_subclass)
+    suppress_message(linter, _leave_class(MisdesignChecker), 'R0904', is_model_admin_subclass)
 
     # Tests
-    suppress_message(linter, MisdesignChecker.leave_class, 'R0904', is_model_test_case_subclass)
+    suppress_message(linter, _leave_class(MisdesignChecker), 'R0904', is_model_test_case_subclass)
 
     # View
     # Method could be a function (get, post)
-    suppress_message(linter, ClassChecker.leave_function, 'R0201', is_model_view_subclass_method_shouldnt_be_function)
+    suppress_message(linter, _leave_function(ClassChecker), 'R0201', is_model_view_subclass_method_shouldnt_be_function)
     # Unused argument 'request' (get, post)
-    suppress_message(linter, VariablesChecker.leave_function, 'W0613', is_model_view_subclass_unused_argument)
+    suppress_message(linter, _leave_function(VariablesChecker), 'W0613', is_model_view_subclass_unused_argument)
 
     # django-mptt
-    suppress_message(linter, DocStringChecker.visit_class, 'C0111', is_model_mpttmeta_subclass)
-    suppress_message(linter, NewStyleConflictChecker.visit_class, 'C1001', is_model_mpttmeta_subclass)
-    suppress_message(linter, ClassChecker.visit_class, 'W0232', is_model_mpttmeta_subclass)
-    suppress_message(linter, MisdesignChecker.leave_class, 'R0903', is_model_mpttmeta_subclass)
+    suppress_message(linter, _visit_class(DocStringChecker), 'missing-docstring', is_model_mpttmeta_subclass)
+    suppress_message(linter, _visit_class(NewStyleConflictChecker), 'old-style-class', is_model_mpttmeta_subclass)
+    suppress_message(linter, _visit_class(ClassChecker), 'W0232', is_model_mpttmeta_subclass)
+    suppress_message(linter, _leave_class(MisdesignChecker), 'too-few-public-methods', is_model_mpttmeta_subclass)
 
     # ForeignKey and OneToOneField
     VariablesChecker.leave_module = wrap(VariablesChecker.leave_module, ignore_import_warnings_for_related_fields)
